//@line 37 "/home/ondrejd/mozilla/extensions/xpcomviewer/resources/content/bookmarks-mozstorage.js"

/**
 * Description:
 * ============
 * In this file you can find all whats needed for bookmarks functionality 
 * in the XPCOMViewer. It not use RDF datasources as the database for longer 
 * but new mozStorage with embedded SQLite instead. This is why this code 
 * can be used only on Mozilla products based on Gecko 1.9.x and above.
 * (Firefox 2 for example).
 * Code self is splitted into three main parts (objects):
 *    gBookmarksSrv      				 Main object. Is entry point for using 
 * 															 the two others. It contains methods that 
 *															 dealing with XPCOMViewer's UI events.
 *    gBookmarksDb       				 Object for dealing with bookmarks database.
 *    BookmarksTreeViewPrototype Object deriven from TreeViewPrototype in 
 *                               global.js file.
 */

///////////////////////////////////////////////////////////////////////////////
// Global variables (some are initialized in gXpcomViewer.onLoad() or 
// in gXpcomViewer.delayedLoad())

var gBookmarksTree = null;
var gDbConnect = null; // mozIStorageConnection



/**
 * Returns new object of a single XPCOMViewer's bookmark.
 *
 * @param   integer ID of bookmark
 * @param   string  Title of bookmark
 * @param   string  Bookmark's notes
 * @param   string  Bookmark's target
 * @param   string  Since bookmark is created or last edited
 * @return  object
 */
function SingleBookmarkObjectPrototype(aId,
                                       aTitle,
                                       aNotes,
                                       aTarget,
                                       aSince)
{
  this.mId     = aId;
  this.mTitle  = aTitle;
  this.mNotes  = aNotes;
  this.mTarges = aTarget;
  this.mSince  = aSince;
  
  return this;
}



///////////////////////////////////////////////////////////////////////////////
// Bookmarks database object


/**
 * Object for dealing with bookmarks database.
 */
var gBookmarksDb = 
{

  /**
   * Add bookmark to the database
   * 
   * @param  object  Bookmark to add
   * @return boolean Returns TRUE if adding completed successfuly or FALSE
   */
  add : function(aBookmark) 
  {
    if(!gDbConnect) return false;
    
    var stmt = gDbConnect.createStatement("INSERT INTO bookmarks " + 
        "(title, notes, target, since) VALUES (?1, ?2, ?3, ?4) ;");
    stmt.bindUTF8StringParameter(0, aBookmark.mTitle);
    stmt.bindUTF8StringParameter(1, aBookmark.mNotes);
    stmt.bindStringParameter(2, aBookmark.mTarget);
    stmt.bindStringParameter(3, aBookmark.mSince);
    stmt.execute();
    return true;
  }, // end gBookmarksDb.add(aBookmark)
  

  /**
   * Gets single bookmark object with given id from the database
   * 
   * @param  string  Bookmark's identifier
   * @return mixed   Returns bookmark object or FALSE
   */
  get : function(aBookmarkId) 
  {
    if(!gDbConnect) return false;
    
    var bookmark = false;
    var stmt = gDbConnect.createStatement("SELECT * FROM bookmarks " + 
        "WHERE ID = ?1 LIMIT 1 ;");
    stmt.bindInt32Parameter(0, aBookmarkId);
    
    while(stmt.executeStep()) {
      bookmark = new SingleBookmarkObjectPrototype(stmt.getInt32(0),
                                                   stmt.getUTF8String(1),
                                                   stmt.getUTF8String(2),
                                                   stmt.getUTF8String(3),
                                                   stmt.getUTF8String(4));
    }
    
    return bookmark;
  }, // end gBookmarksDb.get(aBookmarkId)
  
  
  /**
   * Gets single bookmark object by the given bookmark's title
   * 
   * @param  string  Bookmarks's title
   * @return mixed   Returns bookmark object or FALSE
   */
  getByTitle : function(aBookmarkTitle)
  {
    if(!gDbConnect) return false;
    
    var bookmark = false;
    var stmt = gDbConnect.createStatement("SELECT * FROM bookmarks " + 
        "WHERE title = ?1 LIMIT 1 ;");
    stmt.bindUTF8StringParameter(0, aBookmarkTitle);
    
    while(stmt.executeStep()) {
      bookmark = new SingleBookmarkObjectPrototype(stmt.getInt32(0),
                                                   stmt.getUTF8String(1),
                                                   stmt.getUTF8String(2),
                                                   stmt.getUTF8String(3),
                                                   stmt.getUTF8String(4));
    }
    
    return bookmark;
  }, // end gBookmarksDb.getByTitle(aBookmarkTitle)
  

  /**
   * Gets single bookmark object by the given target
   * 
   * @param  string  Bookmark's target
   * @return mixed   Returns bookmark object or FALSE
   */
  getByTarget : function(aBookmarkTarget) 
  {
    if(!gDbConnect) return false;
    
    var bookmark = false;
    var stmt = gDbConnect.createStatement("SELECT * FROM bookmarks " + 
        "WHERE target = ?1 LIMIT 1 ;");
    stmt.bindUTF8StringParameter(0, aBookmarkTarget);
    
    while(stmt.executeStep()) {
      bookmark = new SingleBookmarkObjectPrototype(stmt.getInt32(0),
                                                   stmt.getUTF8String(1),
                                                   stmt.getUTF8String(2),
                                                   stmt.getUTF8String(3),
                                                   stmt.getUTF8String(4));
    }
    
    return bookmark;
  }, // end gBookmarksDb.getByTarget(aBookmarkTarget)
  
  
  /**
   * Returns array with all bookmarks that are stored in the database.
   * 
   * @return  array
   */
  getAll : function()
  {
		if(!gDbConnect) return false;
    
    var bookmarks = [];
    var stmt = gDbConnect.createStatement("SELECT * FROM bookmarks " + 
        "ORDER BY title ASC ;");
    
    while(stmt.executeStep()) {
      var bookmark = new SingleBookmarkObjectPrototype(stmt.getInt32(0),
                                                       stmt.getUTF8String(1),
                                                       stmt.getUTF8String(2),
                                                       stmt.getUTF8String(3),
                                                       stmt.getUTF8String(4));
      
  	  bookmarks.push(bookmark);
    }
    
    return bookmarks;
  }, // end gBookmarksDb.getAll()
  
  
  /**
   * Remove bookmark with the given identifier from the database
   * 
   * @param  int     Bookmark's identifier
   * @return mixed   Returns object of deleted bookmark or FALSE
   */
  remove : function(aBookmarkId)
  {
		if(!gDbConnect) return false;
    
    var bookmark = this.get(aBookmarkId);
    if(!(bookmark)) {
      gStatusbar.update(gStrBundle.getString("removeBookmarkError2"), "error");
      return false;
    }
    
    var stmt = gDbConnect.createStatement("DELETE FROM bookmarks " + 
        "WHERE ID = ?1 ;");
    stmt.bindInt32Parameter(0, aBookmarkId);
    stmt.execute();
    
    gStatusbar.update(gStrBundle.getFormattedString("bookmarkRemoveSuccess", [aId]), "success");
    
    return bookmark;
  }, // end gBookmarksDb.remove(aBookmarkId)
  

  /**
   * Remove all bookmarks. 
   */
  removeAll : function()
  {
    var stmt = gDbConnect.createStatement("DELETE FROM bookmarks ");
    stmt.execute();
    
    gStatusbar.update(gStrBundle.getFormattedString("bookmarksRemoveSuccess"), "success");
  }, // end gBookmarksDb.removeAll()
  
  
  /**
   * Update bookmark with the given new one
   * 
   * @param  object  Old bookmark (bookmark to update)
   * @param  object  New bookmark
   * @return boolean Returns TRUE if update succeed
   */
  update : function(aOldBookmark, aNewBookmark)
  {
    if(!gDbConnect) return false;
    
    var stmt = gDbConnect.createStatement("UPDATE bookmarks SET " + 
        "title = ?1, notes = ?2, target = ?3, since = ?4 WHERE ID = ?5 ;");
    stmt.bindUTF8StringParameter(0, aNewBookmark.mTitle);
    stmt.bindUTF8StringParameter(1, aNewBookmark.mNotes);
    stmt.bindUTF8StringParameter(2, aNewBookmark.mTarget);
    stmt.bindUTF8StringParameter(3, aNewBookmark.mSince);
    stmt.bindInt32Parameter(4, aOldBookmark.mId);
    stmt.execute();
    
    return true;
  } // end gBookmarksDb.update(aOldBookmark, aNewBookmark)

}; // End of gBookmarksDb



///////////////////////////////////////////////////////////////////////////////
// Bookmarks tree view object 

/** 
 * Holds informations about if tree is sorted and how. Sort direction holds 
 * -1 for descending direction, 0 for natural direction, 1 for ascending.
 */
var gBookmarksTreeSort = 
{
  sortedCol : "", // Contains name of currently sorted column
  direction : 0,  // -1 is descending, 0 is natural, 1 is ascending
  fastIndex : 0,  // Holds count of tree items to examine if tree's row count 
                  // is changed from last sort (if not we can use simple array 
                  // reverse)
  
  /**
   * Method for sorting itself.
   */
  sort : function(aX, aY)
  {
    var xText = aX.mColumns[gBookmarksTreeSort.sortedCol].mCellText.toLowerCase();
    var yText = aY.mColumns[gBookmarksTreeSort.sortedCol].mCellText.toLowerCase();
    
    if(xText > yText) return 1;
    if(xText < yText) return -1;
    return 0;
  }
};

/**
 * Creates new tree item for bookmarks tree
 */
function BookmarksTreeItemPrototype(aBookmark, aLevel, aIsContainer)
{
  var item = new TreeViewItemPrototype(
  		[{ mColId : "idCol", mColText : aBookmark.mId },
  		 { mColId : "descriptionCol", mColText : aBookmark.mDescription },
  		 { mColId : "targetCol", mColText : aBookmark.mTarget },
  		 { mColId : "sinceCol", mColText : aBookmark.mSince }],
  		 aLevel, aIsContainer);

	return item;
}

/**
 * Implementation of tree view used in our bookmarks tree. Derived from 
 * the common TreeViewPrototype.
 * @see global.js
 */
function BookmarksTreeViewPrototype(aTreeView)
{
  var treeView = new TreeViewPrototype(aTreeView);
  
  treeView.cycleHeader = function (aCol)
  {
    var index = gBookmarksTree.currentIndex;
    
    if(aCol.id == gBookmarksTreeSort.sortedCol)
      gBookmarksTreeSort.direction = (gBookmarksTreeSort.direction === 0) 
          ? 1 : -gBookmarksTreeSort.direction;
    
    if(aCol.id == gBookmarksTreeSort.sortedCol && 
       gBookmarksTreeSort.fastIndex == this.rowCount)
    {
      this.__treeView__.reverse();
    }
    else
    {
      if(gBookmarksTreeSort.sortedCol != "") {
        var old = document.getElementById(gBookmarksTreeSort.sortedCol);
        old.setAttribute("sortDirection", "");
      }
      gBookmarksTreeSort.sortedCol = aCol.id;
      this.__treeView__.sort(gBookmarksTreeSort.sort);
    }
    
    var sortDir = (gBookmarksTreeSort.sortDirection == 1) 
        ? "ascending" : "descending";
    aCol.element.setAttribute("sortDirection", sortDir);
    this.treebox.invalidate();
    
    if(index >= 0) {
      this.selection.select(index);
      this.treebox.ensureRowIsVisible(index);
    }
    
    gBookmarksTreeSort.fastIndex = this.rowCount;
  };
  
  return treeView;
} // End of BookmarksTreeViewPrototype(aTreeView)

/**
 * Fired when user doubleklicked on the bookmarks tree. Scrolls to target
 * and (if needed) switch the view
 * @param  nsIEvent
 */
function onBookmarkTreeDblClick(aEvent)
{
  var bookmarkDesc = gBookmarksTree.view.getCellText(gBookmarksTree.currentIndex, 
      gBookmarksTree.columns.getPrimaryColumn());
  var bookmark = gBookmarksDb.getByItsDescription(bookmarkDesc);
  
  var targetView = "";
  var targetRow = -1;
  var searchView = null;
  
  var searchFunc = function(aTargetName, aTargetView) {
        var searchView = gXpcomTreeViewBackup[aTargetView];
        for(var i=0; i<searchView.length; i++) {
          if(aTargetName == searchView[i].mColumns["nameCol"].mCellText) 
            return i;
        }
        return false;
      };
  
  var res;
  if(bookmark.mTarget.indexOf("NS_")===0) {
    // shortcut for results view
    res = searchFunc(bookmark.mTarget, "results");
    if(res!=false) {
      targetView = "results";
      targetRow = res;
    }
  } else if(bookmark.mTarget.indexOf("@")==0) {
    // shortcut for classes view
    res = searchFunc(bookmark.mTarget, "classes");
    if(res!=false) {
      targetView = "classes";
      targetRow = res;
    }
  } else {
    // search all views one by one
    res = searchFunc(bookmark.mTarget, "classes");
    if(res!=false) {
      targetView = "classes";
      targetRow = res;
    }
    
    if(targetRow < 0) {
      res = searchFunc(bookmark.mTarget, "ifaces");
      if(res!=false) {
        targetView = "ifaces";
        targetRow = res;
      }
    }
    
    if(targetRow < 0) {
      res = searchFunc(bookmark.mTarget, "results");
      if(res!=false) {
        targetView = "results";
        targetRow = res;
      }
    }
  }

  if (targetRow >= 0 && targetView != "") {
    gXpcomViewer.refreshTree(targetView);
    gXpcomTree.treeBoxObject.scrollToRow (targetRow);
    gXpcomTree.treeBoxObject.view.selection.select (targetRow);
  }
} // End onBookmarkTreeDblClick(aEvent)


///////////////////////////////////////////////////////////////////////////////
// Main bookmarks service


/**
 * Main bookmarks service. This is an entry point for all what you wan to do 
 * with bookmarks.
 */
var gBookmarksSrv = 
{
  
  /**
   * Init bookmarks service. 
   */
  init : function ()
  {
    this.refreshTree();
    gAllowBookmarks = true;
  }, // end init()
  
  /**
   * Refresh bookmarks tree
   */
  refreshTree : function ()
  {
    var treeItems = [];
    var bookmarks = gBookmarksDb.getAll();
    for(var i=0; i < bookmarks.length; i++) 
      treeItems.push(BookmarksTreeItemPrototype(bookmarks[i], 0, false));
    
    gBookmarksTree.view = new BookmarksTreeViewPrototype(treeItems); 
  }, // end refreshTree()

  /**
   * Returns FALSE if target name hasn't attached bookmark or 
   * ID of attached bookmark.
   * @param  string  Bookmark's target
   * @return mixed
   */
  hasItemBookmark : function(aTarget)
  {
    if(!gDbConnect) return false;
    
    var bookmark = gBookmarksDb.getByTarget(aTarget);
    return (!(bookmark)) ? false : bookmark.mId;
  }, // end hasItemBookmark(aTarget)
  
	/**
	 * Add bookmark 
	 */
	addBookmark : function()
	{
	  var targetName = gXpcomTree.view.getCellText(gXpcomTree.currentIndex, 
		    gXpcomTree.columns.getPrimaryColumn());
		
    try {
  	  if(gBookmarksSrv.hasItemBookmark(targetName)) {
        gStatusbar.update(gStrBundle.getString("bookmarkAddWarning"), "warning");
  	    return;
  	  }
  	  
  	  var dlg = window.openDialog("chrome://xpcomviewer/content/notedialog.xul", 
  	      "xpcomviewer:noteDialog", 
  	      "chrome,extrachrome,dialog,modal,resizable=no,centerscreen", 
  	      ["bookmark", "add", targetName]);
  	  dlg.focus();
    } catch (e) {
      gStatusbar.update(gStrBundle.getString("bookmarkAddError"));
      Components.utils.reportError(e);
    }
	}, // end addBookmark()
  
  /**
   * Edit bookmark
   * @param  string
   */
  editBookmark : function(aId)
  {
    var bookmark = gBookmarksDb.get(aId);
    if(!bookmark) {
      gStatusbar.update(gStrBundle.getFormattedString("bookmarkEditError",
          [bookmarkName]), "error");
      return;
    }
  	  
    var dlg = window.openDialog("chrome://xpcomviewer/content/notedialog.xul", 
        "xpcomviewer:noteDialog", 
        "chrome,extrachrome,dialog,modal,resizable=no,centerscreen", 
        ["bookmark", "edit", bookmark]);
    dlg.focus();
  }, // end editBookmark(aId)

	/**
	 * Remove bookmark
   * @param  string
	 */
	removeBookmark : function(aId)
	{
	  var res = gBookmarksDb.remove(aId);
	  if (!(res)) {
      gStatusbar.update(gStrBundle.getString("bookmarkRemoveError1"), "error");
    }
    
    this.refreshTree();
	} // end removeBookmark(aId)
  
}; // End of gBookmarksSrv
